# -*- coding: utf-8 -*-
import sys
import os
import yaml
import datetime
import re

sys.path.append(os.path.abspath("."))
from .ngajKeyType import NgajKeyType

kt = NgajKeyType()

class NgajConfig:
    def __init__(self):
        self.config = None
        self.configfile = None
        self.condition = []
        self.order = {"site":[], "source":[], "smrec":[]}
        self.orderAll = []
        self.outHeader = {}

    def load(self, filepath):
        self.configfile = filepath
        try:
            with open(self.configfile, 'r') as yml:
                self.config = yaml.safe_load(yml)
        except Exception as e:
            print(str(e))
            print("Config File Read Error:%s" % (self.configfile))
            sys.exit(1)

    def parse(self):
        try:
            if self.config == None:
                self.outHeader = kt.columnName
                return
            for mainkey in self.config:
                if mainkey == "source" or mainkey == "site" or mainkey == "smrec":
                    if self.config[mainkey] == None:
                        continue
                    for subkey in list(self.config[mainkey].keys()):
                        if subkey in kt.keyTypeNum:
                            if 'min' in self.config[mainkey][subkey]:
                                if self.config[mainkey][subkey]["min"] is not None:
                                    if not self.is_num(self.config[mainkey][subkey]["min"]):
                                        errmsg = "Config Value Error:key={},value={}".format(subkey, self.config[mainkey][subkey]["min"])
                                        raise ValueError(errmsg)
                                    text = str(float(self.config[mainkey][subkey]["min"])) + " <= "\
                                        + mainkey + "." + subkey
                                    self.condition.append(text)
                            if 'max' in self.config[mainkey][subkey]:
                                if self.config[mainkey][subkey]["max"] is not None:
                                    if not self.is_num(self.config[mainkey][subkey]["max"]):
                                        errmsg = "Config Value Error:key={},value={}".format(subkey, self.config[mainkey][subkey]["max"])
                                        raise ValueError(errmsg)
                                    text = mainkey + "." + subkey + " <= "\
                                        + str(float(self.config[mainkey][subkey]["max"])) 
                                    self.condition.append(text)
                        elif subkey in kt.keyTypeDate:
                            if 'min' in self.config[mainkey][subkey]:                            
                                if self.config[mainkey][subkey]["min"] is not None:
                                    if not self.is_dateformat(self.config[mainkey][subkey]["min"]):
                                        errmsg = "Config Value Error:key={},value={}".format(subkey, self.config[mainkey][subkey]["min"])
                                        raise Exception(errmsg)
                                    text = "'" + str(self.config[mainkey][subkey]["min"]) + "'" + " <= "\
                                        + subkey
                                    self.condition.append(text)
                            if 'max' in self.config[mainkey][subkey]:
                                if self.config[mainkey][subkey]["max"] is not None:
                                    if not self.is_dateformat(self.config[mainkey][subkey]["max"]):
                                        errmsg = "Config Value Error:key={},value={}".format(subkey, self.config[mainkey][subkey]["max"])
                                        raise Exception(errmsg)
                                    text = subkey + " <= "\
                                        + "'"+ str(self.config[mainkey][subkey]["max"]) + "'"
                                    self.condition.append(text)
                        elif subkey in kt.keyTypeId:
                            if self.config[mainkey][subkey] is not None:
                                if "," not in str(self.config[mainkey][subkey]):
                                    if not self.is_int(self.config[mainkey][subkey]):
                                        errmsg = "Config Value Error:key={},value={}".format(subkey, self.config[mainkey][subkey])
                                        raise ValueError(errmsg)
                                    text = subkey + " = " + str(int(self.config[mainkey][subkey]))
                                    self.condition.append(text)
                                else:
                                    text = "("
                                    for i, configvalue in enumerate(str(self.config[mainkey][subkey]).split(",")):
                                        if not self.is_int(configvalue):
                                            errmsg = "Config Value Error:key={},value={}".format(subkey, configvalue)
                                            raise ValueError(errmsg)
                                        if i != 0:
                                            text = text + " OR "
                                        text = text + subkey + " = " + str(int(configvalue))
                                    text = text + ")"
                                    self.condition.append(text)
                        elif subkey in kt.keyTypeName:
                            if self.config[mainkey][subkey] is not None:
                                if "," not in str(self.config[mainkey][subkey]):
                                    configvalue = str(self.config[mainkey][subkey]).strip(" ")
                                    text = subkey + " = " + "'" + str(self.config[mainkey][subkey]) + "'"
                                    self.condition.append(text)
                                else:
                                    text = "("
                                    for i, configvalue in enumerate(str(self.config[mainkey][subkey]).split(",")):
                                        configvalue = configvalue.strip(" ")
                                        if i != 0:
                                            text = text + " OR "
                                        text = text + subkey + " = " + "'" + configvalue + "'"
                                    text = text + ")"
                                    self.condition.append(text)
                        else:
                            errmsg = "Key Error:{} {}".format(mainkey, subkey)
                            raise Exception(errmsg)
                elif mainkey == "sort":
                    if self.config[mainkey] == None:
                        continue                    
                    elif self.config[mainkey] is not None:
                        for configvalue in str(self.config[mainkey]).split(","):
                            configvalue = configvalue.strip(" ")
                            if " " in configvalue:
                                l = re.split(" +", configvalue)
                                if (l[0] in kt.columnNameAll) and (l[1] == "DESC" or l[1] == "ASC"):
                                    if l[0] in kt.columnName["site"]:
                                        self.order["site"].append(configvalue)
                                    if l[0] in kt.columnName["source"]:
                                        self.order["source"].append(configvalue)
                                    if l[0] in kt.columnName["smrec"]:
                                        self.order["smrec"].append(configvalue)
                                    self.orderAll.append(configvalue)
                                else:
                                    errmsg = "Config Value Error:{}".format(configvalue)
                                    raise Exception(errmsg)
                            else:
                                if configvalue in kt.columnNameAll:
                                    if configvalue in kt.columnName["site"]:
                                        self.order["site"].append(configvalue)
                                    if configvalue in kt.columnName["source"]:
                                        self.order["source"].append(configvalue)
                                    if configvalue in kt.columnName["smrec"]:
                                        self.order["smrec"].append(configvalue)
                                    self.orderAll.append(configvalue)
                                else:
                                    errmsg = "Config Value Error:{}".format(configvalue)
                                    raise Exception(errmsg)
                elif mainkey == "column":
                    if self.config[mainkey] == None:
                        continue                    
                    for subkey in list(self.config[mainkey].keys()):
                        tmp = []
                        for column in self.config[mainkey][subkey]:
                            if column in kt.duplicateColumnName:
                                tmp.append(column)
                            elif column in kt.columnName[subkey]:
                                tmp.append(column)
                            else:
                                errmsg = "Config Value Error:{}[{}]".format(subkey, column)
                                raise Exception(errmsg)
                        self.outHeader[subkey] = tmp
                else:
                    errmsg = "Key Error:{}".format(mainkey)
                    raise Exception(errmsg)
            if len(self.outHeader) == 0:
                self.outHeader = kt.columnName
        except Exception as e:
            print(str(e))
            print("Config File Parse Error:{}".format(self.configfile))
            sys.exit(1)
    
    def getCondition(self):
        return self.condition

    def getOrder(self):
        return self.order

    def getOrderAll(self):
        return self.orderAll

    def getHeader(self):
        return self.outHeader

    def is_num(self, s):
        try:
            float(s)
        except:
            return False
        return True

    def is_int(self, s):
        if not str(s).isdigit():
            return False
        return True

    def is_dateformat(self, s):
        if not isinstance(s, datetime.datetime):
            return False
        return True
