# -*- coding: utf-8 -*-

import sys
import os
import sqlite3

sys.path.append(os.path.abspath("."))
from .ngajKeyType import NgajKeyType

kt = NgajKeyType()

class NgajDB:
    def __init__(self, dbname):
        self.conn = None
        self.cur = None
        self.data = {"site":[], "source":[], "smrec":[]}
        self.header = {"site":[], "source":[], "smrec":[]}

        try:
            self.conn = sqlite3.connect(dbname)
            self.cur = self.conn.cursor()
        except Exception as e:
            print(str(e))
            sys.exit(1)

    def __del__(self):
        try:
            self.cur.close()
            self.conn.close()
        except Exception as e:
            print(str(e))
            sys.exit(1)

    def mkConditionSQL(self, condition):
        try:
            s = " WHERE "
            for i, item in enumerate(condition):
                if i != 0:
                    s += "AND "
                s += item + " "
        except Exception as e:
            print(str(e))
            sys.exit(1)
        return s

    def mkOrderSQL(self, order):
        try:
            s = " ORDER BY "
            for i, item in enumerate(order):
                if i != 0:
                    s += ","
                s += item + " "
            if s == " ORDER BY ":
                s = ""
        except Exception as e:
            print(str(e))
            sys.exit(1)
        return s

    def selectDBAll(self, condition, order, header):
        s = ""
        i = 0
        keyStartIndex = {}
        keyEndIndex = {}

        grpflg = True

        for items in header:
            keyStartIndex[items] = i
            for column in header[items]:
                if column in kt.segmentColumnName:
                    grpflg = False
                if i != 0:
                    s += ", "
                s += "cast(" + items + "." + column + " as text)"
                i += 1
            keyEndIndex[items] = i

        sql = "SELECT {} FROM smrec\
                    LEFT JOIN site ON smrec.siteid2 = site.siteid2\
                    LEFT JOIN source ON smrec.eq_source_id = source.eq_source_id"\
                    .format(s)

        if condition:
            sql += self.mkConditionSQL(condition)
        if grpflg:
            sql += " GROUP BY smrec_id"
        if order:
            sql += self.mkOrderSQL(order)

        try:
            for c in self.cur.execute(sql):
                for key in keyEndIndex:
                    self.data[key].append(c[keyStartIndex[key]:keyEndIndex[key]])
        except Exception as e:
            print(str(e))
            sys.exit(1)

    def selectDBPart(self, condition, order, header):

        grpflg = False
        for items in header:
            s = ""
            i = 0
            if items == "smrec":
                grpflg = True
            for column in header[items]:
                if column in kt.segmentColumnName:
                    grpflg = False
                if i != 0:
                    s += ", "
                s += "cast(" + items + "." + column + " as text)"
                i += 1

            sql = "SELECT {} FROM smrec\
                    LEFT JOIN site ON smrec.siteid2 = site.siteid2\
                    LEFT JOIN source ON smrec.eq_source_id = source.eq_source_id"\
                    .format(s)

            if condition:
                sql += self.mkConditionSQL(condition)
            if grpflg:
                sql += " GROUP BY smrec_id"
            if items in order:
                sql += self.mkOrderSQL(order[items])

            try:
                for c in self.cur.execute(sql):
                    self.data[items].append(c)
            except Exception as e:
                print(str(e))
                sys.exit(1)

    def selectHeader(self):
        try:
            for h in self.cur.execute('PRAGMA table_info(\'smrec\')'):
                self.header["smrec"].append(h[1])
            for h in self.cur.execute('PRAGMA table_info(\'site\')'):
                self.header["site"].append(h[1])
            for h in self.cur.execute('PRAGMA table_info(\'source\')'):
                self.header["source"].append(h[1])
        except Exception as e:
            print(str(e))
            sys.exit(1)

    def getSelectData(self):
        return self.data

    def getHeader(self):
        return self.header
