# -*- coding: utf-8 -*-
import argparse
import os
import sys
import sqlite3

from lib.ngajFlatFile import NgajFlatFile

if __name__ == '__main__':
    # Arguments check
    argp = argparse.ArgumentParser(description='NGAJ Database initialize tool')
    argp.add_argument('--input', help='<site_file> <source file> <smrec_file>', nargs=3, type=argparse.FileType('r'), metavar=('<site.tsv>', '<source.tsv>', '<smrec.tsv>'), required=True)
    argp.add_argument('--db', help='SQLite database file name', metavar='<DB FILE>', required=True)
    args = argp.parse_args()
    
    dbfile = args.db
    # check db file exists
    if(os.path.isfile(dbfile)):
        print("ERROR: Database file {} alredy exists.".format(dbfile))
        sys.exit(0)
        
    ff = NgajFlatFile()
    rec_list = ['site', 'source', 'smrec']
    conn = None
    status = False
    try:
        conn = sqlite3.connect(dbfile,isolation_level='EXCLUSIVE')
        ff.make_schema(conn)
        for i, recid in enumerate(rec_list):
            filename = args.input[i].name
            inserted = ff.insert_table(conn,recid,filename)
        status = True
    except Exception as e:
        print("ERROR: create or insert table failed: ", e)
        status = False
        if conn: 
            conn.rollback()
    finally:
        if conn: 
            conn.commit()
    
    if conn:
        conn.close()
    
    if status:
        print('SUCCESS INITDB: {}'.format(dbfile))
        sys.exit(1)
    else:
        sys.exit(0)

