# -*- coding: utf-8 -*-

import argparse
import os
import sys

from lib.ngajDB import NgajDB
from lib.ngajCsv import NgajCsv
from lib.ngajKeyType import NgajKeyType
from lib.ngajConfig import NgajConfig

def __args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--db', help="データベースファイル名", required=True)
    parser.add_argument('--conf', help="設定ファイル名", required=True)
    parser.add_argument('--output', help="出力ファイル識別子", required=True)
    parser.add_argument('--all', help="出力スイッチ", action="store_const", const=True, default=False)
    return parser.parse_args()

if __name__ == '__main__':
    parsed = __args()

    print("--db:{}".format(parsed.db))
    print("--conf:{}".format(parsed.conf))
    print("--output:{}".format(parsed.output))
    print("--all:{}".format(parsed.all))

    if parsed.db:
        dbfile = str(parsed.db)
    if parsed.conf:
        conffile = str(parsed.conf)
    if parsed.output:
        identifier = str(parsed.output)

    if not os.path.isfile(dbfile):
        print("Database file is not found:{}".format(dbfile))
        sys.exit(1)
    if not os.path.isfile(conffile):
        print("CONF file is not found:{}".format(conffile))
        sys.exit(1)

    if parsed.all:
        outputAllCsvFile = "all_schema_" + identifier + ".csv"
        if os.path.isfile(outputAllCsvFile):
            print("Output csv file alredy exists:{}".format(outputAllCsvFile))
            sys.exit(1)
    else:
        outputSiteCsvFile = "site_schema_" + identifier + ".csv"
        if os.path.isfile(outputSiteCsvFile):
            print("Output site csv file alredy exists:{}".format(outputSiteCsvFile))
            sys.exit(1)
        outputSourceCsvFile = "source_schema_" + identifier + ".csv"
        if os.path.isfile(outputSourceCsvFile):
            print("Output source csv file alredy exists:{}".format(outputSourceCsvFile))
            sys.exit(1)
        outputSmrecCsvFile = "smrec_schema_" + identifier + ".csv"
        if os.path.isfile(outputSmrecCsvFile):
            print("Output smrec csv file alredy exists:{}".format(outputSmrecCsvFile))
            sys.exit(1)

    ncfg = NgajConfig()
    ncfg.load(conffile)
    ncfg.parse()
    condition = ncfg.getCondition()
    header = ncfg.getHeader()

    ndb = NgajDB(dbfile)
    if parsed.all:
        order = ncfg.getOrderAll()
        ndb.selectDBAll(condition, order, header)
    else:
        order = ncfg.getOrder()
        ndb.selectDBPart(condition, order, header)
    data = ndb.getSelectData()

    ncsv = NgajCsv(identifier, data, header)
    if parsed.all:
        ncsv.outputAll(outputAllCsvFile)
    else:
        ncsv.outputPart(outputSiteCsvFile, outputSourceCsvFile, outputSmrecCsvFile)
    
    sys.exit(0)
